"use strict";

var chai = require('chai');

var chaiAsPromised = require('chai-as-promised');

var nock = require('nock');

var fs = require('fs');

var _require = require('./FileReaders'),
    LocalFileReader = _require.LocalFileReader,
    RemoteFileReader = _require.RemoteFileReader;

chai.use(chaiAsPromised);
var expect = chai.expect;
describe('LocalFileReader', function () {
  it('should read specific bytes', function () {
    var reader = new LocalFileReader('./test-data/single_sample.vcf.gz.tbi.uncompressed');
    return reader.bytes(1, 4).then(function (buffer) {
      var magic = Buffer.from([0x42, 0x49, 0x01, 0x01]);
      expect(Buffer.compare(Buffer.from(buffer), magic)).to.equal(0);
    });
  });
  it('should read entire file', function () {
    var testFile = './test-data/single_sample.vcf.gz.tbi.uncompressed';
    var reader = new LocalFileReader(testFile);
    return reader.bytes().then(function (buffer) {
      var contents = fs.readFileSync(testFile); // returns Buffer

      expect(Buffer.compare(Buffer.from(buffer), contents)).to.equal(0);
    });
  });
});
describe('RemoteFileReader', function () {
  beforeEach(function () {
    nock.cleanAll();
  });
  it('should read specific bytes', function () {
    var url = 'http://example.com/single_sample.vcf.gz.tbi.uncompressed';
    var contents = Buffer.from([0x42, 0x49, 0x01, 0x01]);
    var index = nock('http://example.com', {
      reqheaders: {
        range: 'bytes=1-4'
      }
    }).get('/single_sample.vcf.gz.tbi.uncompressed').reply(200, contents);
    var reader = new RemoteFileReader(url);
    return reader.bytes(1, 4).then(function (buffer) {
      expect(Buffer.compare(Buffer.from(buffer), contents)).to.equal(0);
      index.done();
    });
  });
  it('should read specific bytes from start to end of file', function () {
    var url = 'http://example.com/single_sample.vcf.gz.tbi.uncompressed';
    var contents = fs.readFileSync('./test-data/single_sample.vcf.gz.tbi.uncompressed').slice(1);
    var index = nock('http://example.com', {
      reqheaders: {
        range: 'bytes=1-'
      }
    }).get('/single_sample.vcf.gz.tbi.uncompressed').reply(200, contents);
    var reader = new RemoteFileReader(url);
    return reader.bytes(1).then(function (buffer) {
      expect(Buffer.compare(Buffer.from(buffer), contents)).to.equal(0);
      index.done();
    });
  });
  it('should read entire file', function () {
    var url = 'http://example.com/single_sample.vcf.gz.tbi.uncompressed';
    var file = './test-data/single_sample.vcf.gz.tbi.uncompressed';
    var contents = fs.readFileSync(file);
    var index = nock('http://example.com').get('/single_sample.vcf.gz.tbi.uncompressed').replyWithFile(200, file);
    var reader = new RemoteFileReader(url);
    return reader.bytes().then(function (buffer) {
      expect(Buffer.compare(Buffer.from(buffer), contents)).to.equal(0);
      index.done();
    });
  });
});